@@ -78,6 +78,57 @@ impl<'a> ConstantReader<'a> {
     }
 }
 
+pub struct ConstantEditor<'a> {
+    constants: &'a mut [f32],
+    offset: usize,
+}
+
+impl<'a> ConstantEditor<'a> {
+    pub fn new(constants: &'a mut [f32]) -> Self {
+        Self {
+            constants,
+            offset: 0,
+        }
+    }
+    pub fn skip(&mut self, count: usize) {
+        self.offset += count;
+    }
+    // Specific skips for readability
+    pub fn skip_f32(&mut self) {
+        self.skip(1);
+    }
+    pub fn skip_vec3(&mut self) {
+        self.skip(3);
+    }
+
+    pub fn at_end(&self) -> bool {
+        self.offset == self.constants.len()
+    }
+
+    pub fn edit_f32(&mut self, editor: impl FnOnce(f32) -> f32) -> Result<(), Error> {
+        if let Some(value) = self.constants.get_mut(self.offset) {
+            *value = editor(*value);
+            self.offset += 1;
+            Ok(())
+        } else {
+            Err(Error::BadConstants)
+        }
+    }
+    pub fn edit_vec3(&mut self, editor: impl FnOnce(Vec3) -> Vec3) -> Result<(), Error> {
+        // TODO: I'm sure there's some much better way to write this.
+        if let Some([x, y, z]) = self.constants.get_mut(self.offset..(self.offset + 3)) {
+            let edited = editor(Vec3::new(*x, *y, *z));
+            *x = edited.x;
+            *y = edited.y;
+            *z = edited.z;
+            self.offset += 3;
+            Ok(())
+        } else {
+            Err(Error::BadConstants)
+        }
+    }
+}
+
 fn compile_node(graph: &Graph, root: NodeId, ctx: &mut Program, path: &mut Vec<NodeId>) {
     debug_assert!(!path.contains(&root), "Graph cannot contain cycles!");
 
@@ -542,7 +593,7 @@ pub fn disassemble(opcodes: &[Opcode], constants: &[f32]) -> Result<String, Erro
                 str.push_str(&format!("PushRotation: {}\n", rotation));
             }
             Opcode::PopTransform => {
-                str.push_str("PopTransform\n");
+                str.push_str("Pop\n");
             }
         }
     }
